// CLSDMXCPLUGINIMPL : Implementierung von TclsDMXCPluginImpl (CoClass: clsDMXCPlugin, Schnittstelle: IclsDMXCPlugin)

#include <vcl.h>
#pragma hdrstop

#include "CLSDMXCPLUGINIMPL.H"

/////////////////////////////////////////////////////////////////////////////
// TclsDMXCPluginImpl


// String via USB holen / Konvertierungsfunktion
// aus uDMX - Projekt uebernommen
int TclsDMXCPluginImpl::usbStringGet(usb_dev_handle *dev, int index, 
                               int langid, char *buf, int buflen)
{
  char    buffer[256];
  int     rval, i;

  if((rval = usb_control_msg(dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR, 
                             (USB_DT_STRING << 8) + index, langid, 
                             buffer, sizeof(buffer), 1000)) < 0)
    return rval;

  if(buffer[1] != USB_DT_STRING)
    return 0;

  if((unsigned char)buffer[0] < rval)
    rval = (unsigned char)buffer[0];

  rval /= 2;
  /* lossy conversion to ISO Latin1 */
  for(i=1;i<rval;i++)
  {
    if(i > buflen)  /* destination buffer overflow */
      break;
    buf[i-1] = buffer[2 * i];
    if(buffer[2 * i + 1] != 0)  /* outside of ISO Latin1 range */
      buf[i-1] = '?';
  }
  buf[i-1] = 0;
  return i-1;
}

// Pluginnamen an Anwendung uebermitteln
STDMETHODIMP TclsDMXCPluginImpl::get_PluginName(BSTR* Value)
{
  try
  {
    *Value = wsName.Copy() ;
  }
  catch(Exception &e)
  {
    return Error(e.Message.c_str(), IID_IclsDMXCPlugin);
  }
  return S_OK;
};

// NICHT ERFORDERLICH ===
STDMETHODIMP TclsDMXCPluginImpl::get_ModuleInfo(_Type_Module_Info** Param1)
{
  try
  {

  }
  catch(Exception &e)
  {
    return Error(e.Message.c_str(), IID__IDMXCModule);
  }
  return S_OK;
};

// Konfiguration
STDMETHODIMP TclsDMXCPluginImpl::Configure(VARIANT_BOOL* Param1)
{
  /*
  // ### LH - Do NOT use this way
  // otherwise DLL will depend on Borland DLLs
  Application->MessageBox("No Configuration necessary for uDMX\r\n" \
                          "Written and compiled by ilLUTZminator\r\n" \
                          "[ ilLUTZminator@users.sourceforge.net ]",
                          AnsiString(wsName).c_str(),
                          MB_OK + MB_ICONEXCLAMATION);
  */

  MessageBox (NULL,
              "No Configuration necessary for uDMX\r\n" \
              "Written and compiled by ilLUTZminator\r\n" \
              "[ ilLUTZminator@users.sourceforge.net ]",
              AnsiString(wsName).c_str(),
              MB_OK + MB_ICONEXCLAMATION);

  *Param1 = true ;
  return S_OK;
}

// Das Plugin entfernen, Hardware deaktivieren
STDMETHODIMP TclsDMXCPluginImpl::Disable(VARIANT_BOOL* Param1)
{
  usb_close(m_hUSB) ;
  m_hUSB = NULL ;

  *Param1 = true ;
  return S_OK;
}

// Plugin starten, Hardware aktivieren
STDMETHODIMP TclsDMXCPluginImpl::Enable(VARIANT_BOOL* Param1)
{
  bool  bRes = true ;
  long  Chan, Val ;
  short sRes ;

  m_numOfDevices = 0 ;

  usb_find_busses();
  usb_find_devices();

  /* Search for uDMX device on all busses */
  for(bus = usb_busses; bus; bus = bus->next)
  {
    for(dev = bus->devices; dev; dev = dev->next)
    {
      if((dev->descriptor.idVendor  == USBDEV_SHARED_VENDOR) &&
         (dev->descriptor.idProduct == USBDEV_SHARED_PRODUCT))
      {
        char    string[256];
        int     len;

        /* we need to open the device in order to query strings */
        m_hUSB = usb_open(dev);
        if(!m_hUSB)
        {
          continue;
        }

        /* now find out whether the device is uDMX */
        len = usbStringGet(m_hUSB, dev->descriptor.iManufacturer, 0x0409, string, sizeof(string));
        if(len < 0)
        {
          goto skipDevice;
        }

        if(strcmp(string, "www.anyma.ch") != 0)
        {
          goto skipDevice;
        }

        len = usbStringGet(m_hUSB, dev->descriptor.iProduct, 0x0409, string, sizeof(string));
        if(len < 0)
        {
          goto skipDevice;
        }

        if(strcmp(string, "uDMX") != 0)
        {
skipDevice:
          usb_close(m_hUSB) ;
          m_hUSB = NULL ;
        }
        else
        {
          // uDMX device found !
          m_numOfDevices ++;
          goto LoopEnd ;
        }
      }
    }
  }

LoopEnd:

  if(!m_numOfDevices)
  {
    bRes = false ;
    /*
    // ### LH - Do NOT use this way
    // otherwise DLL will depend on Borland DLLs
    Application->MessageBox("No uDMX-hardware found",
                            AnsiString(wsName).c_str(),
                            MB_OK + MB_ICONSTOP) ;
    */
    MessageBox(NULL,
               "No uDMX-hardware found",
               AnsiString(wsName).c_str(),
               MB_OK + MB_ICONSTOP) ;
  }

  // auf 512 Kanal Modus schalten
  Chan = 512 ;
  Val  = 0 ;
  SetChannel(&Chan, &Val, &sRes) ;

  *Param1 = bRes ;
  return S_OK;
}

// Initialisierung
// Die Strings merken ...
STDMETHODIMP TclsDMXCPluginImpl::Init(BSTR* CallingAppTitle,
  BSTR* MyFileName, _IPluginHelper** DMXCPluginHelper,
  VARIANT_BOOL* Param4)
{
  wsApp = *CallingAppTitle ;
  wsFile = *MyFileName ;

  wsName = wsName + " [" + wsFile + "]" ;

  usb_set_debug(1);
  usb_init();

  *Param4= true ;
  return S_OK;
}

// NICHT ERFORDERLICH ===
STDMETHODIMP TclsDMXCPluginImpl::MessageHandle(MSGs* Msg, long* Arg1,
  long* Arg2, long* Param4)
{
  *Param4= true ;
  return S_OK;
}

//
STDMETHODIMP TclsDMXCPluginImpl::Term(VARIANT_BOOL* Param1)
{ 
  *Param1= true ;
  return S_OK;
}

// DIE entscheidende Funktion
// Wert auf Kanal einstellen
// HINWEIS: uDMX beginnt bei 0 / DmxControl bei 1
STDMETHODIMP TclsDMXCPluginImpl::SetChannel(long* Channel, long* Value,
  VARIANT_BOOL* Param3)
{
  int   nBytes ;
  bool  bRes = true ;
  // Kanalzaehlung angleichen
  int   nKanal  = *Channel - 1 ;

  if (m_hUSB)
  {
    nBytes = usb_control_msg(m_hUSB,
                             USB_TYPE_VENDOR |
                             USB_RECIP_DEVICE |
                             USB_ENDPOINT_OUT,
                             cmd_SetChannelRange,
                             1, nKanal,
                            (char *)Value, 1, 50000);
    if (nBytes < 0)
      bRes = false ;
  }
  else
    bRes = false ;

  *Param3= bRes ;
  return S_OK;
}

// =========== AB HIER: ALLE NICHT BENOETIGT ===================
STDMETHODIMP TclsDMXCPluginImpl::AuthenticationStage1(BSTR* Param1)
{
  return S_OK;
}

STDMETHODIMP TclsDMXCPluginImpl::AuthenticationStage2(BSTR* Response,
  VARIANT_BOOL* Param2)
{
  return S_OK;
}

STDMETHODIMP TclsDMXCPluginImpl::AuthenticationStage3(BSTR* Challenge,
  BSTR* Param2)
{ 
  return S_OK;
}

STDMETHODIMP TclsDMXCPluginImpl::AuthenticationStage4(
  VARIANT_BOOL* Success, VARIANT_BOOL* Param2)
{ 
  return S_OK;
}

STDMETHODIMP TclsDMXCPluginImpl::ChannelGetValue(long* Channel,
  long* Param2)
{ 
  return S_OK;
}

STDMETHODIMP TclsDMXCPluginImpl::ChannelIsDimmer(long* Channel,
  VARIANT_BOOL* Param2)
{ 
  return S_OK;
}

STDMETHODIMP TclsDMXCPluginImpl::DMXInSetChannel(long* Channel,
  long* Value, VARIANT_BOOL* Param3)
{ 
  return S_OK;
}

STDMETHODIMP TclsDMXCPluginImpl::get_ChannelMAXCHANNELS(long* Param1)
{
  try
  {

  }
  catch(Exception &e)
  {
    return Error(e.Message.c_str(), IID__IPluginHelper);
  }
  return S_OK;
};


STDMETHODIMP TclsDMXCPluginImpl::get_Devices(_IDevices** Param1)
{
  try
  {

  }
  catch(Exception &e)
  {
    return Error(e.Message.c_str(), IID__IPluginHelper);
  }
  return S_OK;
};


STDMETHODIMP TclsDMXCPluginImpl::get_DEVisKey(BSTR* Param1)
{
  try
  {

  }
  catch(Exception &e)
  {
    return Error(e.Message.c_str(), IID__IPluginHelper);
  }
  return S_OK;
};


STDMETHODIMP TclsDMXCPluginImpl::get_HighResTimer(_ITimerEx** Param1)
{
  try
  {

  }
  catch(Exception &e)
  {
    return Error(e.Message.c_str(), IID__IPluginHelper);
  }
  return S_OK;
};


STDMETHODIMP TclsDMXCPluginImpl::get_Messages(_IMessages** Param1)
{
  try
  {

  }
  catch(Exception &e)
  {
    return Error(e.Message.c_str(), IID__IPluginHelper);
  }
  return S_OK;
};


STDMETHODIMP TclsDMXCPluginImpl::get_MEVPKey(BSTR* Param1)
{
  try
  {

  }
  catch(Exception &e)
  {
    return Error(e.Message.c_str(), IID__IPluginHelper);
  }
  return S_OK;
};


STDMETHODIMP TclsDMXCPluginImpl::get_MyStream(_IDMXMixerChannel** Param1)
{
  try
  {

  }
  catch(Exception &e)
  {
    return Error(e.Message.c_str(), IID__IPluginHelper);
  }
  return S_OK;
};


STDMETHODIMP TclsDMXCPluginImpl::get_Project(_IProject** Param1)
{
  try
  {

  }
  catch(Exception &e)
  {
    return Error(e.Message.c_str(), IID__IPluginHelper);
  }
  return S_OK;
};


STDMETHODIMP TclsDMXCPluginImpl::get_Settings(_ISettings** Param1)
{
  try
  {

  }
  catch(Exception &e)
  {
    return Error(e.Message.c_str(), IID__IPluginHelper);
  }
  return S_OK;
};


STDMETHODIMP TclsDMXCPluginImpl::get_Storage(_IDataStorage** Param1)
{
  try
  {

  }
  catch(Exception &e)
  {
    return Error(e.Message.c_str(), IID__IPluginHelper);
  }
  return S_OK;
};


STDMETHODIMP TclsDMXCPluginImpl::get_Translator(_ITranslator** Param1)
{
  try
  {

  }
  catch(Exception &e)
  {
    return Error(e.Message.c_str(), IID__IPluginHelper);
  }
  return S_OK;
};


STDMETHODIMP TclsDMXCPluginImpl::GetLocalSetting(BSTR* Key,
  VARIANT* Default, BSTR* Param3)
{ 
  return S_OK;
}

STDMETHODIMP TclsDMXCPluginImpl::SaveLocalSetting(BSTR* Key, BSTR* Setting)
{
  return S_OK;
}



